@@ -11,13 +11,22 @@ from typing import Any, Dict, List, Optional, Sequence, Set, Tuple, Union
 import mcp
 import pydantic
 import pydantic_ai.models
+from dbos import DBOS, SetWorkflowID
 from pydantic_ai import Agent as PydanticAgent
-from pydantic_ai import BinaryContent, DocumentUrl, ImageUrl
-from pydantic_ai import RunContext, UsageLimitExceeded, UsageLimits
+from pydantic_ai import (
+    BinaryContent,
+    DocumentUrl,
+    ImageUrl,
+    RunContext,
+    UsageLimitExceeded,
+    UsageLimits,
+)
+from pydantic_ai.durable_exec.dbos import DBOSAgent
 from pydantic_ai.messages import (
     ModelMessage,
     ModelRequest,
     TextPart,
+    ThinkingPart,
     ToolCallPart,
     ToolCallPartDelta,
     ToolReturn,
@@ -32,11 +41,12 @@ from code_puppy.config import (
     get_compaction_strategy,
     get_compaction_threshold,
     get_global_model_name,
+    get_message_limit,
     get_openai_reasoning_effort,
     get_protected_token_count,
+    get_use_dbos,
     get_value,
     load_mcp_server_configs,
-    get_message_limit,
 )
 from code_puppy.mcp_ import ServerConfig, get_mcp_manager
 from code_puppy.messaging import (
@@ -53,6 +63,11 @@ from code_puppy.model_factory import ModelFactory
 from code_puppy.summarization_agent import run_summarization_sync
 from code_puppy.tools.common import console
 
+# Global flag to track delayed compaction requests
+_delayed_compaction_requested = False
+
+_reload_count = 0
+
 
 class BaseAgent(ABC):
     """Base class for all agent configurations."""
@@ -505,6 +520,98 @@ class BaseAgent(ABC):
             # Be safe; don't blow up status/compaction if model lookup fails
             return 128000
 
+    def has_pending_tool_calls(self, messages: List[ModelMessage]) -> bool:
+        """
+        Check if there are any pending tool calls in the message history.
+
+        A pending tool call is one that has a ToolCallPart without a corresponding
+        ToolReturnPart. This indicates the model is still waiting for tool execution.
+
+        Returns:
+            True if there are pending tool calls, False otherwise
+        """
+        if not messages:
+            return False
+
+        tool_call_ids: Set[str] = set()
+        tool_return_ids: Set[str] = set()
+
+        # Collect all tool call and return IDs
+        for msg in messages:
+            for part in getattr(msg, "parts", []) or []:
+                tool_call_id = getattr(part, "tool_call_id", None)
+                if not tool_call_id:
+                    continue
+
+                if part.part_kind == "tool-call":
+                    tool_call_ids.add(tool_call_id)
+                elif part.part_kind == "tool-return":
+                    tool_return_ids.add(tool_call_id)
+
+        # Pending tool calls are those without corresponding returns
+        pending_calls = tool_call_ids - tool_return_ids
+        return len(pending_calls) > 0
+
+    def request_delayed_compaction(self) -> None:
+        """
+        Request that compaction be attempted after the current tool calls complete.
+
+        This sets a global flag that will be checked during the next message
+        processing cycle to trigger compaction when it's safe to do so.
+        """
+        global _delayed_compaction_requested
+        _delayed_compaction_requested = True
+        emit_info(
+            "🔄 Delayed compaction requested - will attempt after tool calls complete",
+            message_group="token_context_status",
+        )
+
+    def should_attempt_delayed_compaction(self) -> bool:
+        """
+        Check if delayed compaction was requested and it's now safe to proceed.
+
+        Returns:
+            True if delayed compaction was requested and no tool calls are pending
+        """
+        global _delayed_compaction_requested
+        if not _delayed_compaction_requested:
+            return False
+
+        # Check if it's now safe to compact
+        messages = self.get_message_history()
+        if not self.has_pending_tool_calls(messages):
+            _delayed_compaction_requested = False  # Reset the flag
+            return True
+
+        return False
+
+    def get_pending_tool_call_count(self, messages: List[ModelMessage]) -> int:
+        """
+        Get the count of pending tool calls for debugging purposes.
+
+        Returns:
+            Number of tool calls waiting for execution
+        """
+        if not messages:
+            return 0
+
+        tool_call_ids: Set[str] = set()
+        tool_return_ids: Set[str] = set()
+
+        for msg in messages:
+            for part in getattr(msg, "parts", []) or []:
+                tool_call_id = getattr(part, "tool_call_id", None)
+                if not tool_call_id:
+                    continue
+
+                if part.part_kind == "tool-call":
+                    tool_call_ids.add(tool_call_id)
+                elif part.part_kind == "tool-return":
+                    tool_return_ids.add(tool_call_id)
+
+        pending_calls = tool_call_ids - tool_return_ids
+        return len(pending_calls)
+
     def prune_interrupted_tool_calls(
         self, messages: List[ModelMessage]
     ) -> List[ModelMessage]:
@@ -595,6 +702,21 @@ class BaseAgent(ABC):
         compaction_strategy = get_compaction_strategy()
 
         if proportion_used > compaction_threshold:
+            # RACE CONDITION PROTECTION: Check for pending tool calls before summarization
+            if compaction_strategy == "summarization" and self.has_pending_tool_calls(
+                messages
+            ):
+                pending_count = self.get_pending_tool_call_count(messages)
+                emit_warning(
+                    f"⚠️  Summarization deferred: {pending_count} pending tool call(s) detected. "
+                    "Waiting for tool execution to complete before compaction.",
+                    message_group="token_context_status",
+                )
+                # Request delayed compaction for when tool calls complete
+                self.request_delayed_compaction()
+                # Return original messages without compaction
+                return messages, []
+
             if compaction_strategy == "truncation":
                 # Use truncation instead of summarization
                 protected_tokens = get_protected_token_count()
@@ -603,7 +725,7 @@ class BaseAgent(ABC):
                 )
                 summarized_messages = []  # No summarization in truncation mode
             else:
-                # Default to summarization
+                # Default to summarization (safe to proceed - no pending tool calls)
                 result_messages, summarized_messages = self.summarize_messages(
                     self.filter_huge_messages(messages)
                 )
@@ -635,7 +757,6 @@ class BaseAgent(ABC):
                         f"Final token count after processing: {final_token_count}",
                         message_group="token_context_status",
                     )
-
             self.set_message_history(result_messages)
             for m in summarized_messages:
                 self.add_compacted_message_hash(self.hash_message(m))
@@ -852,6 +973,9 @@ class BaseAgent(ABC):
             instructions += f"\n{puppy_rules}"
 
         mcp_servers = self.load_mcp_servers()
+        emit_info(
+            f"[dim]DEBUG: Loaded {len(mcp_servers)} MCP servers during reload[/dim]"
+        )
 
         model_settings_dict: Dict[str, Any] = {"seed": 42}
         output_tokens = max(
@@ -866,16 +990,18 @@ class BaseAgent(ABC):
             model_settings_dict["openai_reasoning_effort"] = (
                 get_openai_reasoning_effort()
             )
-            model_settings_dict["extra_body"] = {"verbosity": "low"}
             model_settings = OpenAIChatModelSettings(**model_settings_dict)
 
+        if model_name.startswith("claude-code"):
+            instructions = "You are Claude Code, Anthropic's official CLI for Claude."
+
         self.cur_model = model
         p_agent = PydanticAgent(
             model=model,
             instructions=instructions,
             output_type=str,
             retries=3,
-            mcp_servers=mcp_servers,
+            toolsets=mcp_servers,
             history_processors=[self.message_history_accumulator],
             model_settings=model_settings,
         )
@@ -883,12 +1009,114 @@ class BaseAgent(ABC):
         agent_tools = self.get_available_tools()
         register_tools_for_agent(p_agent, agent_tools)
 
-        self._code_generation_agent = p_agent
+        # Get existing tool names to filter out conflicts with MCP tools
+        existing_tool_names = set()
+        try:
+            # Get tools from the agent to find existing tool names
+            tools = getattr(p_agent, "_tools", None)
+            if tools:
+                existing_tool_names = set(tools.keys())
+        except Exception:
+            # If we can't get tool names, proceed without filtering
+            pass
+
+        # Filter MCP server toolsets to remove conflicting tools
+        filtered_mcp_servers = []
+        if mcp_servers and existing_tool_names:
+            for mcp_server in mcp_servers:
+                try:
+                    # Get tools from this MCP server
+                    server_tools = getattr(mcp_server, "tools", None)
+                    if server_tools:
+                        # Filter out conflicting tools
+                        filtered_tools = {}
+                        for tool_name, tool_func in server_tools.items():
+                            if tool_name not in existing_tool_names:
+                                filtered_tools[tool_name] = tool_func
+
+                        # Create a filtered version of the MCP server if we have tools
+                        if filtered_tools:
+                            # Create a new toolset with filtered tools
+                            from pydantic_ai.tools import ToolSet
+
+                            filtered_toolset = ToolSet()
+                            for tool_name, tool_func in filtered_tools.items():
+                                filtered_toolset._tools[tool_name] = tool_func
+                            filtered_mcp_servers.append(filtered_toolset)
+                        else:
+                            # No tools left after filtering, skip this server
+                            pass
+                    else:
+                        # Can't get tools from this server, include as-is
+                        filtered_mcp_servers.append(mcp_server)
+                except Exception:
+                    # Error processing this server, include as-is to be safe
+                    filtered_mcp_servers.append(mcp_server)
+        else:
+            # No filtering needed or possible
+            filtered_mcp_servers = mcp_servers if mcp_servers else []
+
+        if len(filtered_mcp_servers) != len(mcp_servers):
+            emit_info(
+                f"[dim]Filtered {len(mcp_servers) - len(filtered_mcp_servers)} conflicting MCP tools[/dim]"
+            )
+
         self._last_model_name = resolved_model_name
         # expose for run_with_mcp
-        self.pydantic_agent = p_agent
+        # Wrap it with DBOS, but handle MCP servers separately to avoid serialization issues
+        global _reload_count
+        _reload_count += 1
+        if get_use_dbos():
+            # Don't pass MCP servers to the agent constructor when using DBOS
+            # This prevents the "cannot pickle async_generator object" error
+            # MCP servers will be handled separately in run_with_mcp
+            agent_without_mcp = PydanticAgent(
+                model=model,
+                instructions=instructions,
+                output_type=str,
+                retries=3,
+                toolsets=[],  # Don't include MCP servers here
+                history_processors=[self.message_history_accumulator],
+                model_settings=model_settings,
+            )
+
+            # Register regular tools (non-MCP) on the new agent
+            agent_tools = self.get_available_tools()
+            register_tools_for_agent(agent_without_mcp, agent_tools)
+
+            # Wrap with DBOS
+            dbos_agent = DBOSAgent(
+                agent_without_mcp, name=f"{self.name}-{_reload_count}"
+            )
+            self.pydantic_agent = dbos_agent
+            self._code_generation_agent = dbos_agent
+
+            # Store filtered MCP servers separately for runtime use
+            self._mcp_servers = filtered_mcp_servers
+        else:
+            # Normal path without DBOS - include filtered MCP servers in the agent
+            # Re-create agent with filtered MCP servers
+            p_agent = PydanticAgent(
+                model=model,
+                instructions=instructions,
+                output_type=str,
+                retries=3,
+                toolsets=filtered_mcp_servers,
+                history_processors=[self.message_history_accumulator],
+                model_settings=model_settings,
+            )
+            # Register regular tools on the agent
+            agent_tools = self.get_available_tools()
+            register_tools_for_agent(p_agent, agent_tools)
+
+            self.pydantic_agent = p_agent
+            self._code_generation_agent = p_agent
+            self._mcp_servers = filtered_mcp_servers
+            self._mcp_servers = mcp_servers
         return self._code_generation_agent
 
+    # It's okay to decorate it with DBOS.step even if not using DBOS; the decorator is a no-op in that case.
+    @DBOS.step()
     def message_history_accumulator(self, ctx: RunContext, messages: List[Any]):
         _message_history = self.get_message_history()
         message_history_hashes = set([self.hash_message(m) for m in _message_history])
@@ -902,6 +1130,14 @@ class BaseAgent(ABC):
         # Apply message history trimming using the main processor
         # This ensures we maintain global state while still managing context limits
         self.message_history_processor(ctx, _message_history)
+        result_messages_filtered_empty_thinking = []
+        for msg in self.get_message_history():
+            if len(msg.parts) == 1:
+                if isinstance(msg.parts[0], ThinkingPart):
+                    if msg.parts[0].content == "":
+                        continue
+            result_messages_filtered_empty_thinking.append(msg)
+            self.set_message_history(result_messages_filtered_empty_thinking)
         return self.get_message_history()
 
     async def run_with_mcp(
@@ -928,7 +1164,12 @@ class BaseAgent(ABC):
         """
         group_id = str(uuid.uuid4())
         # Avoid double-loading: reuse existing agent if already built
-        pydantic_agent = self._code_generation_agent or self.reload_code_generation_agent()
+        pydantic_agent = (
+            self._code_generation_agent or self.reload_code_generation_agent()
+        )
+        if self.get_model_name().startswith("claude-code"):
+            if len(self.get_message_history()) == 0:
+                prompt = self.get_system_prompt() + "\n\n" + prompt
 
         # Build combined prompt payload when attachments are provided.
         attachment_parts: List[Any] = []
@@ -950,13 +1191,64 @@ class BaseAgent(ABC):
                 self.set_message_history(
                     self.prune_interrupted_tool_calls(self.get_message_history())
                 )
+
+                # DELAYED COMPACTION: Check if we should attempt delayed compaction
+                if self.should_attempt_delayed_compaction():
+                    emit_info(
+                        "🔄 Attempting delayed compaction (tool calls completed)",
+                        message_group="token_context_status",
+                    )
+                    current_messages = self.get_message_history()
+                    compacted_messages, _ = self.compact_messages(current_messages)
+                    if compacted_messages != current_messages:
+                        self.set_message_history(compacted_messages)
+                        emit_info(
+                            "✅ Delayed compaction completed successfully",
+                            message_group="token_context_status",
+                        )
+
                 usage_limits = UsageLimits(request_limit=get_message_limit())
-                result_ = await pydantic_agent.run(
-                    prompt_payload,
-                    message_history=self.get_message_history(),
-                    usage_limits=usage_limits,
-                    **kwargs,
-                )
+
+                # Handle MCP servers - add them temporarily when using DBOS
+                if (
+                    get_use_dbos()
+                    and hasattr(self, "_mcp_servers")
+                    and self._mcp_servers
+                ):
+                    # Temporarily add MCP servers to the DBOS agent using internal _toolsets
+                    original_toolsets = pydantic_agent._toolsets
+                    pydantic_agent._toolsets = original_toolsets + self._mcp_servers
+                    pydantic_agent._toolsets = original_toolsets + self._mcp_servers
+
+                    try:
+                        # Set the workflow ID for DBOS context so DBOS and Code Puppy ID match
+                        with SetWorkflowID(group_id):
+                            result_ = await pydantic_agent.run(
+                                prompt_payload,
+                                message_history=self.get_message_history(),
+                                usage_limits=usage_limits,
+                                **kwargs,
+                            )
+                    finally:
+                        # Always restore original toolsets
+                        pydantic_agent._toolsets = original_toolsets
+                elif get_use_dbos():
+                    # DBOS without MCP servers
+                    with SetWorkflowID(group_id):
+                        result_ = await pydantic_agent.run(
+                            prompt_payload,
+                            message_history=self.get_message_history(),
+                            usage_limits=usage_limits,
+                            **kwargs,
+                        )
+                else:
+                    # Non-DBOS path (MCP servers are already included)
+                    result_ = await pydantic_agent.run(
+                        prompt_payload,
+                        message_history=self.get_message_history(),
+                        usage_limits=usage_limits,
+                        **kwargs,
+                    )
                 return result_
             except* UsageLimitExceeded as ule:
                 emit_info(f"Usage limit exceeded: {str(ule)}", group_id=group_id)
@@ -972,8 +1264,12 @@ class BaseAgent(ABC):
                 )
             except* asyncio.exceptions.CancelledError:
                 emit_info("Cancelled")
+                if get_use_dbos():
+                    await DBOS.cancel_workflow_async(group_id)
             except* InterruptedError as ie:
                 emit_info(f"Interrupted: {str(ie)}")
+                if get_use_dbos():
+                    await DBOS.cancel_workflow_async(group_id)
             except* Exception as other_error:
                 # Filter out CancelledError and UsageLimitExceeded from the exception group - let it propagate
                 remaining_exceptions = []
