@@ -7,6 +7,7 @@ import time
 import webbrowser
 from pathlib import Path
 
+from dbos import DBOS, DBOSConfig
 from rich.console import Console, ConsoleOptions, RenderResult
 from rich.markdown import CodeBlock, Markdown
 from rich.syntax import Syntax
@@ -14,21 +15,24 @@ from rich.text import Text
 
 from code_puppy import __version__, callbacks, plugins
 from code_puppy.agents import get_current_agent
+from code_puppy.command_line.attachments import parse_prompt_attachments
 from code_puppy.command_line.prompt_toolkit_completion import (
     get_input_with_combined_completion,
     get_prompt_with_active_model,
 )
-from code_puppy.command_line.attachments import parse_prompt_attachments
 from code_puppy.config import (
     AUTOSAVE_DIR,
     COMMAND_HISTORY_FILE,
+    DBOS_DATABASE_URL,
     ensure_config_exists,
     finalize_autosave_session,
+    get_use_dbos,
     initialize_command_history_file,
     save_command_to_history,
 )
-from code_puppy.session_storage import restore_autosave_interactively
 from code_puppy.http_utils import find_available_port
+from code_puppy.messaging import emit_info
+from code_puppy.session_storage import restore_autosave_interactively
 from code_puppy.tools.common import console
 
 # message_history_accumulator and prune_interrupted_tool_calls have been moved to BaseAgent class
@@ -175,15 +179,22 @@ async def main():
 
     # Handle agent selection from command line
     if args.agent:
-        from code_puppy.agents.agent_manager import set_current_agent, get_available_agents
+        from code_puppy.agents.agent_manager import (
+            get_available_agents,
+            set_current_agent,
+        )
 
         agent_name = args.agent.lower()
         try:
             # First check if the agent exists by getting available agents
             available_agents = get_available_agents()
             if agent_name not in available_agents:
-                emit_system_message(f"[bold red]Error:[/bold red] Agent '{agent_name}' not found")
-                emit_system_message(f"Available agents: {', '.join(available_agents.keys())}")
+                emit_system_message(
+                    f"[bold red]Error:[/bold red] Agent '{agent_name}' not found"
+                )
+                emit_system_message(
+                    f"Available agents: {', '.join(available_agents.keys())}"
+                )
                 sys.exit(1)
 
             # Agent exists, set it
@@ -192,7 +203,7 @@ async def main():
         except Exception as e:
             emit_system_message(f"[bold red]Error setting agent:[/bold red] {str(e)}")
             sys.exit(1)
-            
+
     current_version = __version__
 
     no_version_update = os.getenv("NO_VERSION_UPDATE", "").lower() in (
@@ -216,6 +227,32 @@ async def main():
 
     await callbacks.on_startup()
 
+    # Initialize DBOS if not disabled
+    if get_use_dbos():
+        dbos_message = f"Initializing DBOS with database at: {DBOS_DATABASE_URL}"
+        emit_system_message(dbos_message)
+
+        dbos_config: DBOSConfig = {
+            "name": "dbos-code-puppy",
+            "system_database_url": DBOS_DATABASE_URL,
+            "run_admin_server": False,
+            "conductor_key": os.environ.get(
+                "DBOS_CONDUCTOR_KEY"
+            ),  # Optional, if set in env, connect to conductor
+            "log_level": os.environ.get(
+                "DBOS_LOG_LEVEL", "ERROR"
+            ),  # Default to ERROR level to suppress verbose logs
+            "application_version": current_version,  # Match DBOS app version to Code Puppy version
+        }
+        try:
+            DBOS(config=dbos_config)
+            DBOS.launch()
+        except Exception as e:
+            emit_system_message(f"[bold red]Error initializing DBOS:[/bold red] {e}")
+            sys.exit(1)
+    else:
+        emit_system_message("DBOS is disabled. Running without durable execution.")
+
     global shutdown_flag
     shutdown_flag = False
     try:
@@ -258,6 +295,8 @@ async def main():
         if message_renderer:
             message_renderer.stop()
         await callbacks.on_shutdown()
+        if get_use_dbos():
+            DBOS.destroy()
 
 
 # Add the file handling functionality for interactive mode
@@ -279,6 +318,12 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
     emit_system_message(
         "Press [bold red]Ctrl+C[/bold red] during processing to cancel the current task or inference."
     )
+    emit_system_message(
+        "Use [bold blue]/autosave_load[/bold blue] to manually load a previous autosave session."
+    )
+    emit_system_message(
+        "Use [bold blue]/diff[/bold blue] to configure diff highlighting colors for file changes."
+    )
     try:
         from code_puppy.command_line.motd import print_motd
 
@@ -291,7 +336,6 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
 
     emit_info("[bold cyan]Initializing agent...[/bold cyan]")
 
-
     # Initialize the runtime agent manager
     if initial_command:
         from code_puppy.agents import get_current_agent
@@ -312,7 +356,7 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
                 awaiting_input = False
 
             # Run with or without spinner based on whether we're awaiting input
-            response = await run_prompt_with_attachments(
+            response, agent_task = await run_prompt_with_attachments(
                 agent,
                 initial_command,
                 spinner_console=display_console,
@@ -362,7 +406,10 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
             emit_error(f"Error installing prompt_toolkit: {e}")
             emit_warning("Falling back to basic input without tab completion")
 
-    await restore_autosave_interactively(Path(AUTOSAVE_DIR))
+    # Autosave loading is now manual - use /autosave_load command
+
+    # Track the current agent task for cancellation on quit
+    current_agent_task = None
 
     while True:
         from code_puppy.agents.agent_manager import get_current_agent
@@ -397,15 +444,31 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
             "/exit",
             "/quit",
         ]:
+            import asyncio
+
             from code_puppy.messaging import emit_success
 
             emit_success("Goodbye!")
+
+            # Cancel any running agent task for clean shutdown
+            if current_agent_task and not current_agent_task.done():
+                emit_info("Cancelling running agent task...")
+                current_agent_task.cancel()
+                try:
+                    await current_agent_task
+                except asyncio.CancelledError:
+                    pass  # Expected when cancelling
+
             # The renderer is stopped in the finally block of main().
             break
 
         # Check for clear command (supports both `clear` and `/clear`)
         if task.strip().lower() in ("clear", "/clear"):
-            from code_puppy.messaging import emit_info, emit_system_message, emit_warning
+            from code_puppy.messaging import (
+                emit_info,
+                emit_system_message,
+                emit_warning,
+            )
 
             agent = get_current_agent()
             new_session_id = finalize_autosave_session()
@@ -425,14 +488,25 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
                 command_result = handle_command(cleaned_for_commands)
             except Exception as e:
                 from code_puppy.messaging import emit_error
+
                 emit_error(f"Command error: {e}")
                 # Continue interactive loop instead of exiting
                 continue
             if command_result is True:
                 continue
             elif isinstance(command_result, str):
-                # Command returned a prompt to execute
-                task = command_result
+                if command_result == "__AUTOSAVE_LOAD__":
+                    # Handle async autosave loading
+                    try:
+                        await restore_autosave_interactively(Path(AUTOSAVE_DIR))
+                    except Exception as e:
+                        from code_puppy.messaging import emit_error
+
+                        emit_error(f"Failed to load autosave: {e}")
+                    continue
+                else:
+                    # Command returned a prompt to execute
+                    task = command_result
             elif command_result is False:
                 # Command not recognized, continue with normal processing
                 pass
@@ -447,7 +521,7 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
                 # No need to get agent directly - use manager's run methods
 
                 # Use our custom helper to enable attachment handling with spinner support
-                result = await run_prompt_with_attachments(
+                result, current_agent_task = await run_prompt_with_attachments(
                     current_agent,
                     task,
                     spinner_console=message_renderer.console,
@@ -463,15 +537,11 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
                     f"\n[bold purple]AGENT RESPONSE: [/bold purple]\n{agent_response}"
                 )
 
-                # Auto-save session if enabled
-                from code_puppy.config import auto_save_session_if_enabled
-                auto_save_session_if_enabled()
-
                 # Ensure console output is flushed before next prompt
                 # This fixes the issue where prompt doesn't appear after agent response
                 display_console.file.flush() if hasattr(
                     display_console.file, "flush"
-        ) else None
+                ) else None
                 import time
 
                 time.sleep(0.1)  # Brief pause to ensure all messages are rendered
@@ -481,6 +551,11 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
 
                 get_queue_console().print_exception()
 
+            # Auto-save session if enabled (moved outside the try block to avoid being swallowed)
+            from code_puppy.config import auto_save_session_if_enabled
+
+            auto_save_session_if_enabled()
+
 
 def prettier_code_blocks():
     class SimpleCodeBlock(CodeBlock):
@@ -509,7 +584,13 @@ async def run_prompt_with_attachments(
     spinner_console=None,
     use_spinner: bool = True,
 ):
-    """Run the agent after parsing CLI attachments for image/document support."""
+    """Run the agent after parsing CLI attachments for image/document support.
+
+    Returns:
+        tuple: (result, task) where result is the agent response and task is the asyncio task
+    """
+    import asyncio
+
     from code_puppy.messaging import emit_system_message, emit_warning
 
     processed_prompt = parse_prompt_attachments(raw_prompt)
@@ -531,26 +612,37 @@ async def run_prompt_with_attachments(
         emit_warning(
             "Prompt is empty after removing attachments; add instructions and retry."
         )
-        return None
+        return None, None
 
     attachments = [attachment.content for attachment in processed_prompt.attachments]
     link_attachments = [link.url_part for link in processed_prompt.link_attachments]
 
+    # Create the agent task first so we can track and cancel it
+    agent_task = asyncio.create_task(
+        agent.run_with_mcp(
+            processed_prompt.prompt,
+            attachments=attachments,
+            link_attachments=link_attachments,
+        )
+    )
+
     if use_spinner and spinner_console is not None:
         from code_puppy.messaging.spinner import ConsoleSpinner
 
         with ConsoleSpinner(console=spinner_console):
-            return await agent.run_with_mcp(
-                processed_prompt.prompt,
-                attachments=attachments,
-                link_attachments=link_attachments,
-            )
-
-    return await agent.run_with_mcp(
-        processed_prompt.prompt,
-        attachments=attachments,
-        link_attachments=link_attachments,
-    )
+            try:
+                result = await agent_task
+                return result, agent_task
+            except asyncio.CancelledError:
+                emit_info("Agent task cancelled")
+                return None, agent_task
+    else:
+        try:
+            result = await agent_task
+            return result, agent_task
+        except asyncio.CancelledError:
+            emit_info("Agent task cancelled")
+            return None, agent_task
 
 
 async def execute_single_prompt(prompt: str, message_renderer) -> None:
@@ -651,8 +743,10 @@ def main_entry():
     except KeyboardInterrupt:
         # Just exit gracefully with no error message
         callbacks.on_shutdown()
+        if get_use_dbos():
+            DBOS.destroy()
         return 0
 
 
 if __name__ == "__main__":
-    main_entry()
\ No newline at end of file
+    main_entry()
