@@ -15,6 +15,7 @@ from textual.widgets import Footer, ListView
 # message_history_accumulator and prune_interrupted_tool_calls have been moved to BaseAgent class
 from code_puppy.agents.agent_manager import get_current_agent
 from code_puppy.command_line.command_handler import handle_command
+from code_puppy.command_line.model_picker_completion import set_active_model
 from code_puppy.config import (
     get_global_model_name,
     get_puppy_name,
@@ -35,7 +36,13 @@ from code_puppy.tui.components import (
 # Import shared message classes
 from .messages import CommandSelected, HistoryEntrySelected
 from .models import ChatMessage, MessageType
-from .screens import HelpScreen, MCPInstallWizardScreen, SettingsScreen, ToolsScreen
+from .screens import (
+    HelpScreen,
+    MCPInstallWizardScreen,
+    ModelPicker,
+    SettingsScreen,
+    ToolsScreen,
+)
 
 
 class CodePuppyTUI(App):
@@ -176,11 +183,11 @@ class CodePuppyTUI(App):
         # Start the message renderer EARLY to catch startup messages
         # Using call_after_refresh to start it as soon as possible after mount
         self.call_after_refresh(self.start_message_renderer_sync)
-        
+
         # Kick off a non-blocking preload of the agent/model so the
         # status bar shows loading before first prompt
         self.call_after_refresh(self.preload_agent_on_startup)
-        
+
         # After preload, offer to restore an autosave session (like interactive mode)
         self.call_after_refresh(self.maybe_prompt_restore_autosave)
 
@@ -205,7 +212,7 @@ class CodePuppyTUI(App):
             tight_lines = []
             last_blank = False
             for ln in lines:
-                is_blank = (ln == "")
+                is_blank = ln == ""
                 if is_blank and last_blank:
                     continue
                 tight_lines.append(ln)
@@ -489,6 +496,10 @@ class CodePuppyTUI(App):
                     self.app.exit()
                     return
 
+                if message.strip().lower() in ("/model", "/m"):
+                    self.action_open_model_picker()
+                    return
+
                 # Use the existing command handler
                 # The command handler directly uses the messaging system, so we don't need to capture stdout
                 try:
@@ -520,11 +531,9 @@ class CodePuppyTUI(App):
                     self.add_agent_message(agent_response)
 
                     # Auto-save session if enabled (mirror --interactive)
-                    try:
-                        from code_puppy.config import auto_save_session_if_enabled
-                        auto_save_session_if_enabled()
-                    except Exception:
-                        pass
+                    from code_puppy.config import auto_save_session_if_enabled
+
+                    auto_save_session_if_enabled()
 
                     # Refresh history display to show new interaction
                     self.refresh_history_display()
@@ -716,6 +725,22 @@ class CodePuppyTUI(App):
 
         self.push_screen(MCPInstallWizardScreen(), handle_wizard_result)
 
+    def action_open_model_picker(self) -> None:
+        """Open the model picker modal."""
+
+        def handle_model_select(model_name: str | None):
+            if model_name:
+                try:
+                    set_active_model(model_name)
+                    self.current_model = model_name
+                    status_bar = self.query_one(StatusBar)
+                    status_bar.current_model = self.current_model
+                    self.add_system_message(f"✅ Model switched to: {model_name}")
+                except Exception as e:
+                    self.add_error_message(f"Failed to switch model: {e}")
+
+        self.push_screen(ModelPicker(), handle_model_select)
+
     def process_initial_command(self) -> None:
         """Process the initial command provided when starting the TUI."""
         if self.initial_command:
@@ -969,10 +994,12 @@ class CodePuppyTUI(App):
     async def maybe_prompt_restore_autosave(self) -> None:
         """Offer to restore an autosave session at startup (TUI version)."""
         try:
-            import asyncio
             from pathlib import Path
 
-            from code_puppy.config import AUTOSAVE_DIR, set_current_autosave_from_session_name
+            from code_puppy.config import (
+                AUTOSAVE_DIR,
+                set_current_autosave_from_session_name,
+            )
             from code_puppy.session_storage import list_sessions, load_session
 
             base_dir = Path(AUTOSAVE_DIR)
@@ -1032,7 +1059,7 @@ class CodePuppyTUI(App):
             # Use Textual's push_screen with a result callback
             def on_picker_result(result_name=None):
                 # Schedule async handler to avoid blocking UI
-                import asyncio
+
                 self.run_worker(handle_result(result_name), exclusive=False)
 
             self.push_screen(picker, on_picker_result)
