@@ -7,6 +7,7 @@ from .mcp_install_wizard import MCPInstallWizardScreen
 from .settings import SettingsScreen
 from .tools import ToolsScreen
 from .autosave_picker import AutosavePicker
+from .model_picker import ModelPicker
 
 __all__ = [
     "HelpScreen",
@@ -14,4 +15,5 @@ __all__ = [
     "ToolsScreen",
     "MCPInstallWizardScreen",
     "AutosavePicker",
+    "ModelPicker",
 ]
